# evaluate how well vizrank performs as a classifier
# perform test with several changes
import orngRegistry
orngRegistry.addWidgetDirectories()

import orange, time, os, sys, cPickle, orngStat, orngTest
from orngMisc import demangleExamples
from orngVizRank import *

bootstrapDir = os.path.join(os.getcwd(), "bootstrap")
resultsFile = os.path.join(bootstrapDir, "bootstrap-results.txt")
bootstrapFile = os.path.join(bootstrapDir, "bootstrapIndices.txt")

names = ["leukemia.tab", "DLBCL.tab", "prostata.tab", "MLL.tab", "SRBCT.tab", "lung.tab"]

vizrank = VizRank(RADVIZ)
vizrankName = "VizRank"

B = 100
datasetIndices = {}
random.seed(0)

f = open(bootstrapFile, "r")
datasetIndices = cPickle.load(f)
f.close()

if os.path.exists(resultsFile):
    f = open(resultsFile, "r")
    results = cPickle.load(f)
    f.close()
else:
    results = {}


for filename in names:
    print filename
    datasetDir = os.path.join(bootstrapDir, filename[:-4]) # make folders for saving projections
    if not os.path.exists(datasetDir):
        continue
    
    examples = orange.ExampleTable(filename, dontCheckStored = 1)
    n = len(examples)
    allIndices = datasetIndices[filename]
    
    for b in range(B):
        selIndices = allIndices[b*n:(b+1)*n]
        learnset = examples.getitemsref(selIndices)
        testset = examples.getitemsref([i for i in range(n) if i not in selIndices])
        if not learnset or not testset:
            continue
        
        fullProjFName = os.path.join(datasetDir, "B%02d-%s.proj" % (b, filename[:-4]))
        if not os.path.exists(fullProjFName):
            continue

        testset, testweight = demangleExamples(testset)
        res = orngTest.ExperimentResults(1, [vizrankName], learnset.domain.classVar.values.native(), testweight!=0, learnset.domain.classVar.baseValue)
        #results.results = [orngTest.TestedExample(0, int(examples[i].getclass()), 0, examples[i].getweight(weight)) for i in range(len(testset))]

        vizrank.setData(learnset)
        vizrank.load(fullProjFName)
        for ex in testset:
            te = orngTest.TestedExample(0, int(ex.getclass()), 0, ex.getweight(testweight))
            classVal, dist = vizrank.findArguments(ex)
            te.addResult(classVal, dist)
            res.results.append(te)

        datasetResults = results.get(filename, {})
        learnerResults = datasetResults.get(vizrankName, {})
        learnerResults[b] = res
        datasetResults[vizrankName] = learnerResults
        results[filename] = datasetResults
        print "%02d  %.2f\t" % (b, orngStat.CA(res)[0]*100.0)

    print "---------"
    f = open(resultsFile, "w")
    cPickle.dump(results, f)
    f.close()
