# oceni tocnost algoritmov svm, knn, naivni bayes, drevesa na 6 cancer datasetih
# rezultate strani v "accOfDiffferentLearners.txt" fajl
# rezultati ne vkljucujejo tocnosti vizranka
import orngRegistry
orngRegistry.addWidgetDirectories()

from orngMisc import demangleExamples, getobjectname
import orange, orngTest, orngStat, orngTree, sys, orngFSS
import orngSVM, time, math
from orngVizRank import *
import os, orngVisFuncts, cPickle

svm = orngSVM.SVMLearner()
svm.kernel_type = orange.SVMLearner.LINEAR

knn = orange.kNNLearner(k = 10, rankWeight = 1)
bayes = orange.BayesLearner()
tree = orange.C45Learner()

svm.name = "SVM"
knn.name = "kNN"
bayes.name = "Bayes"
tree.name = "Tree"

learners = [svm, knn, tree, bayes]

names = ["leukemia.tab", "DLBCL.tab", "prostata.tab", "MLL.tab", "SRBCT.tab", "lung.tab"]

bootstrapDir = os.path.join(os.getcwd(), "bootstrap")
resultsFile = os.path.join(bootstrapDir, "bootstrap-results.txt")

datasetIndices = {}
random.seed(0)
bootstrapFile = os.path.join(bootstrapDir, "bootstrapIndices.txt")
if os.path.exists(bootstrapFile):
    f = open(bootstrapFile, "r")
    datasetIndices = cPickle.load(f)
    f.close()

if os.path.exists(resultsFile):
    f = open(resultsFile, "r")
    results = cPickle.load(f)
    f.close()
else:
    results = {}

print "               ",
for learner in learners:
    print learner.name + "    ",
print

B = 1
for b in range(B):
    for filename in names:
        print "%2d %15s" % (b, filename[:-4]), "\t", 
        for learner in learners:
            examples = orange.ExampleTable(filename, dontCheckStored = 1)
            n = len(examples)
            allIndices = datasetIndices[filename]
            selIndices = allIndices[b*n:(b+1)*n]

            learnset = examples.getitemsref(selIndices)
            testset = examples.getitemsref([i for i in range(n) if i not in selIndices])
            if not learnset or not testset:  continue
    
            if learner.name == "kNN":           # set k to sqrt of number of examples
                learner.k = int(math.sqrt(len(examples)))
            elif learner.name == "Bayes":       # discretize attributes for naive bayes
                learnset = orange.Preprocessor_discretize(learnset, method=orange.EntropyDiscretization())
                testset = orange.ExampleTable(learnset.domain, testset)

            res = orngTest.learnAndTestOnTestData([learner], learnset, testset)
            datasetResults = results.get(filename, {})
            learnerResults = datasetResults.get(learner.name, {})
            learnerResults[b] = res
            datasetResults[learner.name] = learnerResults
            results[filename] = datasetResults
            f = open(resultsFile, "w")
            cPickle.dump(results, f)
            f.close()

            print "%.2f\t" % (orngStat.CA(res)[0]*100.0),
        print


# evaluate accuracies on learning data
for filename in names:
    print filename,
    datasetDir = os.path.join(bootstrapDir, filename[:-4]) # make folders for saving projections
    if not os.path.exists(datasetDir):
        continue
    
    for learner in learners:
        examples = orange.ExampleTable(filename, dontCheckStored = 1)
        if learner.name == "Bayes":
            examples = orange.Preprocessor_discretize(examples, method=orange.EntropyDiscretization())
        elif learner.name == "kNN":           # set k to sqrt of number of examples
            learner.k = int(math.sqrt(len(examples)))
            
        #res = orngTest.learnAndTestOnLearnData([learner], examples)
        res = orngTest.learnAndTestOnTestData([learner], examples, examples)
        datasetResults = results.get(filename, {})
        learnerResults = datasetResults.get(learner.name, {})
        learnerResults["ALL"] = res
        datasetResults[learner.name] = learnerResults
        results[filename] = datasetResults
        print "%.2f\t" % (orngStat.CA(res)[0]*100.0),
    
        
    fullProjFName = os.path.join(datasetDir, "All-%s.proj" % (filename[:-4]))
    if not os.path.exists(fullProjFName):
        continue

    examples = orange.ExampleTable(filename, dontCheckStored = 1)
    examples, testweight = demangleExamples(examples)
    res = orngTest.ExperimentResults(1, [vizrankName], examples.domain.classVar.values.native(), testweight!=0, examples.domain.classVar.baseValue)

    vizrank.setData(examples)
    vizrank.load(fullProjFName)
    for ex in examples:
        te = orngTest.TestedExample(0, int(ex.getclass()), 0, ex.getweight(testweight))
        classVal, dist = vizrank.findArguments(ex)
        te.addResult(classVal, dist)
        res.results.append(te)

    datasetResults = results.get(filename, {})
    learnerResults = datasetResults.get(vizrankName, {})
    learnerResults["ALL"] = res
    datasetResults[vizrankName] = learnerResults
    results[filename] = datasetResults
    print "%.2f\t" % (orngStat.CA(res)[0]*100.0)

f = open(resultsFile, "w")
cPickle.dump(results, f)
f.close()        