import orngRegistry
orngRegistry.addWidgetDirectories()

import orange
from orngVizRank import *
import os, cPickle

learnerNames = ["VizRank", "SVM", "kNN", "Bayes", "Tree"]
names = ["leukemia.tab", "DLBCL.tab", "prostata.tab", "MLL.tab", "SRBCT.tab", "lung.tab"]
nLrn = len(learnerNames)

bootstrapDir = os.path.join(os.getcwd(), "bootstrap")
resultsFile = os.path.join(bootstrapDir, "bootstrap-results.txt")

datasetIndices = {}
random.seed(0)
bootstrapFile = os.path.join(bootstrapDir, "bootstrapIndices.txt")
if os.path.exists(bootstrapFile):
    f = open(bootstrapFile, "r")
    datasetIndices = cPickle.load(f)
    f.close()

f = open(resultsFile, "r")
results = cPickle.load(f)
f.close()

caText  = "\t\t"
aucText = "\t\t"

#print "         ",
for learner in learnerNames:
    #print learner + "         ",
    caText  += learner + "            "
    aucText += learner + "            "
#print
caText += "\n"
aucText += "\n"


B = 100
ranksCA = [0] * (nLrn)
ranksAUC = [0] * (nLrn)
bootstraps = range(B)
for filename in names:
    #print "%12s\t" % (filename[:-4]),
    caText += "%12s\t" % (filename[:-4])
    aucText += "%12s\t" % (filename[:-4])
    cas = []
    aucs = []
    for learner in learnerNames:
        ca0 = orngStat.CA(results[filename][learner]["ALL"])[0]*100.0
        auc0= orngStat.AUC(results[filename][learner]["ALL"])[0]
        
        c = []
        a = []
        for bootstrap in bootstraps:
            if not results.has_key(filename) or not results[filename].has_key(learner) or not results[filename][learner].has_key(bootstrap):
                continue
            res = results[filename][learner][bootstrap]
            c.append(orngStat.CA(res)[0]*100.0)
            a.append(orngStat.AUC(res)[0])

        # compute average ca and auc
        if len(c) > 0:
            ca = (1-.632)*ca0 + 0.632*statc.mean(c)
            auc= (1-.632)*auc0 + 0.632*statc.mean(a)
            if len(c) > 1:  devCa = statc.std(c)
            else:           devCa = 0
            if len(a) > 1:  devAuc = statc.std(a)
            else:           devAuc = 0
            
        else:
            ca = auc = dev = 0

        #print "%.2f/%.2f\t" % (ca, dev),
        caText += "%.2f/%.2f\t" % (ca, devCa)
        aucText += "%.3f/%.3f\t" % (auc, devAuc)
        cas.append(ca)
        aucs.append(auc)
    #print
    caText += "\n"
    aucText += "\n"

    for i in range(nLrn):
        worse = 0; equal = 0
        for j in range(len(cas)):
            if i == j: continue
            if "%.2f" % cas[i] == "%.2f" % cas[j]: equal += 1
            elif cas[i] < cas[j]: worse += 1
        e = range(1,equal+1)
        ranksCA[i] += 1 + worse + sum(e)/float(len(e)+1)

    for i in range(nLrn):
        worse = 0; equal = 0
        for j in range(len(aucs)):
            if i == j: continue
            if "%.3f" % aucs[i] == "%.3f" % aucs[j]: equal += 1
            elif aucs[i] < aucs[j]: worse += 1
        e = range(1,equal+1)
        ranksAUC[i] += 1 + worse + sum(e)/float(len(e)+1)

caText += "%12s\t" % ("Ranks:")
aucText += "%12s\t" % ("Ranks:")
for i in range(nLrn):
    caText  += "   %.2f         " % (ranksCA[i]/ float(len(names)))
    aucText += "   %.3f        " % (ranksAUC[i]/ float(len(names)))
caText += "\n"
aucText += "\n"

print "\n--------------\n"
print caText
print aucText